//  Copyright 2016 root.ext@gmail.com
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//  http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.

import 'dart:io' as io;
import 'dart:typed_data';

import '../lib/dna.dart';

@Library('Kernel32')
class Kernel32 {
  dynamic lib = new DynamicLibrary<Kernel32>();

  int GetCurrentProcess() {
    return lib.GetCurrentProcess();
  }

  int GetProcessId(@Param(C_INT) int process) {
    return lib.GetProcessId(process);
  }

  int GetModuleFileNameA(
      @Param(C_INT) int process,
      @Param(C_STRING, out: true) Ref<String> imageFileName,
      @Param(C_INT) int size) {
    return lib.GetModuleFileNameA(process, imageFileName, size);
  }

  bool GetProcessWorkingSetSize(
      @Param(C_INT) int process,
      @Param(TYPEDDATA) TypedData minimumWorkingSetSize,
      @Param(TYPEDDATA) TypedData maximumWorkingSetSize) {
    return lib.GetProcessWorkingSetSize(
        process, minimumWorkingSetSize, maximumWorkingSetSize);
  }

  int GetLastError() {
    return lib.GetLastError();
  }
}

void main() {
  Kernel32 kernel32 = new Kernel32();
  var process = kernel32.GetCurrentProcess();
  var processId = kernel32.GetProcessId(process);
  print('pid $processId = ${io.pid}');

  var min = new Uint32List(1);
  var max = new Uint32List(1);
  kernel32.GetProcessWorkingSetSize(process, min, max);
  print('working set min ${min} max ${max} error ${kernel32.GetLastError()}');

  var name = new Ref(new String.fromCharCodes(new List.filled(64, 0)));
  kernel32.GetModuleFileNameA(0, name, name.value.length);
  print('module name  ${name.value} error ${kernel32.GetLastError()}');
}
