//  Copyright 2016 root.ext@gmail.com
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//  http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.

import 'dart:mirrors';
import 'dart:typed_data';

int getInt(ByteData view, int size, int offset) {
  switch (size) {
    case 1:
      return view.getInt8(offset);
    case 2:
      return view.getInt16(offset, Endianness.HOST_ENDIAN);
    case 4:
      return view.getInt32(offset, Endianness.HOST_ENDIAN);
    case 8:
      return view.getInt64(offset, Endianness.HOST_ENDIAN);
    default:
      throw new ArgumentError("Wrong ${#size}");
  }
}

double getFloat(ByteData view, int size, int offset) {
  switch (size) {
    case 4:
      return view.getFloat32(offset);
    case 8:
      return view.getFloat64(offset);
    default:
      throw new ArgumentError("Wrong ${#size}");
  }
}

void setInt(ByteData view, int size, int offset, int value) {
  switch (size) {
    case 1:
      view.setInt8(offset, value);
      break;
    case 2:
      view.setInt16(offset, value, Endianness.HOST_ENDIAN);
      break;
    case 4:
      view.setInt32(offset, value, Endianness.HOST_ENDIAN);
      break;
    case 8:
      view.setInt64(offset, value, Endianness.HOST_ENDIAN);
      break;
    default:
      throw new ArgumentError("Wrong ${#size}");
  }
}

void setFloat(ByteData view, int size, int offset, double value) {
  switch (size) {
    case 4:
      view.setFloat32(offset, value);
      break;
    case 8:
      view.setFloat64(offset, value);
      break;
    default:
      throw new ArgumentError("Wrong ${#size}");
  }
}

dynamic findClassTag(ClassMirror m, Type t) {
  var tag = m.metadata.where((e) => e.type == reflectClass(t)).toList();
  return tag.isNotEmpty ? tag[0] : null;
}

dynamic findMethodTag(MethodMirror m, Type t) {
  var tag = m.metadata.where((e) => e.type == reflectClass(t)).toList();
  return tag.isNotEmpty ? tag[0] : null;
}

dynamic findDeclTag(DeclarationMirror m, Type t) {
  var tag = m.metadata.where((e) => e.type == reflectClass(t)).toList();
  return tag.isNotEmpty ? tag[0] : null;
}

dynamic findParamTag(ParameterMirror m, Type t) {
  var tag = m.metadata.where((e) => e.type == reflectClass(t)).toList();
  return tag.isNotEmpty ? tag[0] : null;
}
