//  Copyright 2016 root.ext@gmail.com
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//  http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.

import 'dart:typed_data';
import 'package:test/test.dart';

import '../lib/dna.dart';
import '../lib/src/coder.dart';

void main() {
  group('struct tests', () {
    setUp(() {});

    tearDown(() {});

    test('typeddata pointer', () {
      var data = new Uint8List(10);
      var pointer1 = getPointer(data);
      var pointer2 = getPointer(data);
      expect(pointer1, equals(pointer2));
    });

    test('sizes', () {
      var sizes = getSizes();
      expect(sizes, isNotEmpty);
    });

    test('struct encode/decode', () {
      var st = new SumSt(1, 2, 3);
      var data = encodeStruct(st);
      expect(data, isNotEmpty);

      var emptySt = new SumSt(0, 0, 0);
      decodeStruct(data, emptySt);
      expect(emptySt.a, equals(1));
      expect(emptySt.b, equals(2));
      expect(emptySt.c, equals(3));
    });

    test('call with args', () {
      var sums = new Sums();
      var res = sums.sum(1, 2);
      expect(res, equals(3));
    });

    test('call with struct', () {
      var sums = new Sums();
      var res = sums.sumSt(new SumSt(1, 2, 3));
      expect(res, equals(6));
    });

    test('call with struct pointer', () {
      var sums = new Sums();
      var res = sums.sumStPtr(new SumSt(1, 2, 3));
      expect(res, equals(6));
    });

//FIXME
/*
    test('call with struct nested', () {
      var sums = new Sums();
      var res =  sums.sumNestStPtr(new SumNestSt(4,5, new SumSt(1,2,3)));
      expect(res, equals(15));
    });
*/
  });
}

@Library('libdna.so', 'dna.dll')
class Sums {
  dynamic lib = new DynamicLibrary<Sums>();

  int sum(@Param(C_INT) int a, @Param(C_INT) int b) {
    return lib.sum(a, b);
  }

  int sumSt(@Param(C_STRUCT) SumSt st) {
    return lib.sumSt(st);
  }

  int sumStPtr(@Param(C_STRUCT) SumSt st) {
    return lib.sumStPtr(st);
  }

  int sumNestStPtr(@Param(C_STRUCT) SumNestSt st) {
    return lib.sumNestStPtr(st);
  }
}

@Struct()
class SumSt {
  @Field(C_CHAR)
  int a;
  @Field(C_INT)
  int b;
  @Field(C_CHAR)
  int c;

  SumSt(this.a, this.b, this.c);
}

@Struct()
class SumNestSt {
  @Field(C_INT)
  int a;
  @Field(C_STRUCT)
  SumSt p;
  @Field(C_INT)
  int b;

  SumNestSt(this.a, this.b, this.p);
}
